@@ -22,6 +22,9 @@ require 'geekast/controllers/lib/callback'
 require 'singleton'
 require 'observer'
 
+# Sig
+require 'geekast/lib/signal'
+
 
 module Geekast
   module Controller
