@@ -1,5 +1,5 @@
-#  callback.rb - 
-#  Copyright (C) 2004-2006 Frédéric Logier
+#  callback.rb -
+#  Copyright (C) 2004-2007 Frédéric Logier
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,6 +18,8 @@
 require 'geekast/views/lib/glade_base'
 require 'geekast/views/lib/multimedia'
 
+# Sig
+require 'geekast/lib/signal'
 
 module Geekast
   module View
@@ -28,6 +30,13 @@ module Geekast
 
       def initialize
         super("geekast.glade", "main")
+        @sig = Sig.instance
+
+        # chat feature disabled
+        # @jabberchat = Chat.new
+
+#        @main.hide_on_delete
+
         bindtextdomain(Geekast::TEXTDOMAIN, nil, nil, "UTF-8")
 
         @icon = GeekastTrayIcon.new(@main)
@@ -42,7 +51,7 @@ module Geekast
 
         @model_yp = TreeviewChannel.new(@treeview_yp)
 
- 
+
         @Vsignal = Hash.new
 
 
@@ -51,12 +60,12 @@ module Geekast
         @model_bookmark = TreeviewBookmark.new(@treeview_bookmark)
 
 
-        @Vsignal["VrefreshBAR"]  = SIGNAL.connect("VrefreshBAR") {|data1, data2, data3|
+        @Vsignal["VrefreshBAR"]  = @sig.connect("VrefreshBAR", ['Integer', 'Integer','Integer']) {|data1, data2, data3|
           refreshBar(data1, data2, data3)
         }
 
 
-        SIGNAL.connect("VclearTreeview") {|data|
+        @sig.connect("VclearTreeview", ['Integer']) {|data|
           case data
           when Config::YP
             @model_yp.clear_tree
@@ -71,66 +80,65 @@ module Geekast
           end
         }
 
-        SIGNAL.connect("VrefreshYP") {|data1,data2,data3|
+        @sig.connect("VrefreshYP", ['Array', 'Integer','Integer']) {|data1,data2,data3|
           puts "VrefreshYP" if $DEBUG
           @icon.append(data1)
           refreshTreeview(@label_yp,@model_yp,data1,data2,data3)
         }
 
-        SIGNAL.connect("VrefreshRelay") {|data1,data2,data3|
+        @sig.connect("VrefreshRelay", ['Array', 'Integer','Integer']) {|data1,data2,data3|
           puts "VrefreshRelay" if $DEBUG
-#          puts data1.class
           @icon.append(data1)
           refreshTreeview(@label_relay,@model_relay,data1,data2,data3)
         }
 
-        SIGNAL.connect("VrefreshConnection") {|data|
+        @sig.connect("VrefreshConnection", ['Array']) {|data|
           puts "VrefreshConnection" if $DEBUG
           refreshTreeview(nil,@model_connections,data,nil,nil)
         }
 
-        SIGNAL.connect("VrefreshInformation") {|data1, data2, data3, data4|
+        @sig.connect("VrefreshInformation", ['Array', 'Array', 'Array', 'Array']) {|data1, data2, data3, data4|
           puts "VrefreshInformation" if $DEBUG
           refreshInformation(data1, data2, data3, data4)
         }
 
-        SIGNAL.connect("VrefreshBM") {|data|
+        @sig.connect("VrefreshBM", ['Array']) {|data|
           refreshBM(data)
         }
 
-        @Vsignal["VnotifyBAR"] = SIGNAL.connect("VnotifyBAR") {|data|
+        @Vsignal["VnotifyBAR"] = @sig.connect("VnotifyBAR", ['String']) {|data|
           notifyBAR(data)
         }
 
-        SIGNAL.connect("VblockSignal") {|data|
+        @sig.connect("VblockSignal", ['String']) {|data|
           @toolbutton_refresh.sensitive = true
-          SIGNAL.handler_block(@Vsignal[data])
+          @sig.handler_block(@Vsignal[data])
         }
 
 
-        SIGNAL.connect("VunblockSignal") {|data|
-          SIGNAL.handler_unblock(@Vsignal[data])
+        @sig.connect("VunblockSignal", ['String']) {|data|
+          @sig.handler_unblock(@Vsignal[data])
         }
 
 
-        SIGNAL.connect("VrefreshVIDEO") {
+        @sig.connect("VrefreshVIDEO", ['Integer']) {
           puts "VrefreshVIDEO" if $DEBUG
-          @notebook_main.page = Config::VIDEO 
+          @notebook_main.page = Config::VIDEO
           @radiobutton_video.active = true
         }
 
 
-        SIGNAL.connect("VstatusChannelinfo") {|data|
+        @sig.connect("VstatusChannelinfo", ['Integer']) {|data|
           puts "VstatusChannelinfo" if $DEBUG
           channelInfo(data)
         }
 
 
-        SIGNAL.connect("VrefreshWeb") {|data|
+        @sig.connect("VrefreshWeb", ['String']) {|data|
           puts "VrefreshWeb" if $DEBUG
           if data
             @gecko.location = data
-            SIGNAL.emit("VnotifyBAR", data + _(" updated"))
+            @sig.emit("VnotifyBAR", data + _(" updated"))
           end
           @notebook_main.page = Config::WEB
           @radiobutton_web.active = true
@@ -139,7 +147,10 @@ module Geekast
 
 
 
+
+
         load_pref
+        @sig.emit("VnotifyBAR", _("first of all, you should clic on 'Yellow Pages' & 'refresh' buttons."))
       end
 
 
@@ -156,63 +167,63 @@ module Geekast
       def refreshInformation(data1, data2, data3, data4)
 
         if data1
-          @label_info_1.text = data1["uptime"]
-          @label_info_2.text = data1["channel_cache"]
-          @label_info_3.text = data1["firewalled"]
-          @label_info_4.text = data1["gnethostcache"]
+          @label_info_1.text = data1[:uptime]
+          @label_info_2.text = data1[:channel_cache]
+          @label_info_3.text = data1[:firewalled]
+          @label_info_4.text = data1[:gnethostcache]
         end
 
         if data2
-          @label_bandwidth_0.text = data2["totalin"]
-          @label_bandwidth_1.text = data2["totalout"]
-          @label_bandwidth_2.text = data2["total"]
-          @label_bandwidth_3.text = data2["internetin"]
-          @label_bandwidth_4.text = data2["internetout"]
-          @label_bandwidth_5.text = data2["internettotal"]
-          @label_bandwidth_6.text = data2["gnetin"]
-          @label_bandwidth_7.text = data2["gnetout"]
-          @label_bandwidth_8.text = data2["gnettotal"]
-          @label_bandwidth_9.text = data2["gnetpacketsin"]
-          @label_bandwidth_10.text = data2["gnetpacketsout"]
-          @label_bandwidth_11.text = data2["gnetpacketstotal"]
+          @label_bandwidth_0.text = data2[:totalin]
+          @label_bandwidth_1.text = data2[:totalout]
+          @label_bandwidth_2.text = data2[:total]
+          @label_bandwidth_3.text = data2[:internetin]
+          @label_bandwidth_4.text = data2[:internetout]
+          @label_bandwidth_5.text = data2[:internettotal]
+          @label_bandwidth_6.text = data2[:gnetin]
+          @label_bandwidth_7.text = data2[:gnetout]
+          @label_bandwidth_8.text = data2[:gnettotal]
+          @label_bandwidth_9.text = data2[:gnetpacketsin]
+          @label_bandwidth_10.text = data2[:gnetpacketsout]
+          @label_bandwidth_11.text = data2[:gnetpacketstotal]
         end
 
         if data3
-          @label_connections_0.text = data3["direct"]
-          @label_connections_1.text = data3["relay"]
-          @label_connections_2.text = data3["cincout"]
-          @label_connections_3.text = data3["pgnu"]
-          @label_connections_4.text = data3["incoming"]
+          @label_connections_0.text = data3[:direct]
+          @label_connections_1.text = data3[:relay]
+          @label_connections_2.text = data3[:cincout]
+          @label_connections_3.text = data3[:pgnu]
+          @label_connections_4.text = data3[:incoming]
 
-          @label_connections_5.text = "Port " + data3["port1"] + " / " + data3["port2"]
-          @label_connections_6.text = data3["port1active"] + " / " + data3["port2active"]
+          @label_connections_5.text = "Port " + data3[:port1] + " / " + data3[:port2]
+          @label_connections_6.text = data3[:port1active] + " / " + data3[:port2active]
 
           #          @label_connections_7.text = data3["port1active"]
           #          @label_connections_8.text = data3["port2active"]
         end
 
         if data4
-          @label_internal_0.text = data4["version"]
-          @label_internal_1.text = data4["rootmsg"]
-          @label_internal_2.text = data4["serverip"]
-          @label_info_0.text = data4["serverip"]
-          @label_internal_3.text = data4["localip"]
-          @label_internal_4.text = data4["serverport"]
-          @label_internal_5.text = data4["serverport1"]
+          @label_internal_0.text = data4[:version]
+          @label_internal_1.text = data4[:rootmsg]
+          @label_internal_2.text = data4[:serverip]
+          @label_info_0.text = data4[:serverip]
+          @label_internal_3.text = data4[:localip]
+          @label_internal_4.text = data4[:serverport]
+          @label_internal_5.text = data4[:serverport1]
         end
 
-        SIGNAL.emit("VnotifyBAR", Config::PEERCAST_IP + _(" updated"))
+        @sig.emit("VnotifyBAR", Config::PEERCAST_IP + _(" updated"))
       end
 
 
       def refreshBM(data)
         puts "VrefreshBM" if $DEBUG
         count = 0
-        while count < data.length
-          refreshTreeview(@label_bookmark,@model_bookmark, data[count]["channel"], count+1, data.length)
-          @icon.append(data[count]["channel"])
-          count += 1
-        end
+        #        while count < data.length
+        refreshTreeview(@label_bookmark,@model_bookmark, data["channel"], count+1, data.length)
+        @icon.append(data["channel"])
+        #          count += 1
+        #        end
       end
 
       def refreshBar(data1, data2, data3)
@@ -246,13 +257,13 @@ module Geekast
             label.set_markup("<b>Channels :</b> " + data2.to_s + " on " + data3.to_s + " ")
           end
           if model == @model_yp && data2 != data3
-            SIGNAL.emit("VnotifyBAR", _(" xml break..."))
+            @sig.emit("VnotifyBAR", _(" xml break..."))
           else
             case model
             when @model_yp
-              SIGNAL.emit("VnotifyBAR", Config::YELLOWPAGES_IP + _(" updated"))
+              @sig.emit("VnotifyBAR", Config::YELLOWPAGES_IP + _(" updated"))
             when @model_connections
-              SIGNAL.emit("VnotifyBAR", Config::PEERCAST_IP + _(" updated"))
+              @sig.emit("VnotifyBAR", Config::PEERCAST_IP + _(" updated"))
             end
           end
       end
@@ -264,7 +275,7 @@ module Geekast
 
 
 
- 
+
 
       def on_showcolumn_activate(widget)
         puts widget.name  if $DEBUG
@@ -367,18 +378,18 @@ module Geekast
       def on_showsidebar_activate(widget)
         if !widget.active?
           @handlebox_sidebar.hide_all
-          @geekastConfig.sidepan_visible = false
+          @geekastConfig.sidepane_visible = false
         else
           @handlebox_sidebar.show_all
-          @geekastConfig.sidepan_visible = true
+          @geekastConfig.sidepane_visible = true
         end
       end
       def on_sidebar_radiobutton_toggled(widget)
         n = 0
-        
+
         # get button list
         buttonList = widget.group
-        
+
         # Parcours de la liste
         while buttonList[n]
           # selected button
@@ -387,7 +398,7 @@ module Geekast
             # puts "button : " + buttonList.length.to_s
             # toto = buttonList.length - n - 1
             # puts "button : " + toto.to_s
-            
+
             #associate toogle button with a notebook page
             @notebook_main.page =  buttonList.length - n - 1
             break
@@ -395,8 +406,24 @@ module Geekast
             n = n + 1
           end
         end
-      end 
-      
+      end
+
+      def on_toggletoolbutton_chat_toggled(button)
+        # Create a new Chat object if the last was destroyed
+        # by a user's quit event. Else we show or hide the chat window
+        if button.active?
+          if @jabberchat.status == 1
+            @jabberchat = Chat.new
+            @jabberchat.show
+          else
+            @jabberchat.show if @jabberchat.status == 0
+          end
+        else
+          @jabberchat.hide if @jabberchat.status == 0
+        end
+
+      end
+
       def on_fullscreen_activate
       end
       def on_connect_activate
@@ -411,10 +438,10 @@ module Geekast
       end
       def on_refresh_activate
         if @notebook_main.page != Config::BOOKMARK && @notebook_main.page != Config::VIDEO
-          SIGNAL.emit("VrefreshBAR", -1, 0, 0)
+          @sig.emit("VrefreshBAR", -1, 0, 0)
           @toolbutton_refresh.sensitive = false
           @icon.clean(@notebook_main.page)
-          SIGNAL.emit("CrefreshNB", @notebook_main.page)
+          @sig.emit("CrefreshNB", @notebook_main.page)
         end
       end
 
@@ -424,10 +451,17 @@ module Geekast
       def on_preferences_activate(widget)
         CallbackPref.new
       end
+
       def on_quit_activate(widget)
+        @multimedia.stop
         Gtk.main_quit
         # $mainloop.quit
       end
+
+      def on_close_activate(widget)
+        @main.hide
+      end
+
       def on_about_activate(widget)
         about_glade = GladeXML.new(Config::DATA_DIR + "/glade/geekast.glade", "about")
         about_glade["about"].hide
@@ -505,8 +539,8 @@ module Geekast
 
         @model_yp.change_filter(6, @comboboxentry_genre.active_text)
 
-        SIGNAL.emit("CrefreshNB", Config::INFORMATION)
-        SIGNAL.emit("CrefreshNB", Config::BOOKMARK)
+        @sig.emit("CrefreshNB", Config::INFORMATION)
+        @sig.emit("CrefreshNB", Config::BOOKMARK)
       end
 
 
@@ -522,7 +556,7 @@ module Geekast
 
 
       def bars_pref
-        if @geekastConfig.sidepan_visible == true
+        if @geekastConfig.sidepane_visible == true
           @checkmenuitem_sidebar.active = true
         else
           @checkmenuitem_sidebar.active = false
@@ -592,12 +626,12 @@ module Geekast
         else
           @age.active = FALSE
         end
-        
+
       end
-      
-      
+
+
     end
-    
+
 
 
 
@@ -606,38 +640,39 @@ module Geekast
       def initialize(url)
         super("geekast.glade", "channel_info")
         @channel_info.hide
+        @sig = Sig.instance
 
-        @SchannelInfo = SIGNAL.connect("VrefreshChannelInfo") {|data|
+        @SchannelInfo = @sig.connect("VrefreshChannelInfo", ['Array']) {|data|
           puts "VrefreshChannelInfo" if $DEBUG
           if data
-            @label_name.text = data["name"]
-            @label_cgenre.text = data["cgenre"]
-            @label_desc.text = data["desc"]
-            @label_url.text = data["url"]
-            @label_comment.text = data["comment"]
-            @label_id.text = data["id"]
-            @label_type.text = data["type"]
-            @label_bitrate.text = data["bitrate"]
-            @label_source.text = data["source"]
-            @label_uptime.text = data["uptime"]
-            @label_status.text = data["status"]
-            @label_position.text = data["position"]
-            @label_head.text = data["head"]
-            @label_artist.text = data["artist"]
-            @label_title.text = data["title"]
-            @label_album.text = data["album"]
-            @label_tgenre.text = data["tgenre"]
-            @label_contact.text = data["contact"]
+            @label_name.text = data[:name]
+            @label_cgenre.text = data[:cgenre]
+            @label_desc.text = data[:desc]
+            @label_url.text = data[:url]
+            @label_comment.text = data[:comment]
+            @label_id.text = data[:id]
+            @label_type.text = data[:type]
+            @label_bitrate.text = data[:bitrate]
+            @label_source.text = data[:source]
+            @label_uptime.text = data[:uptime]
+            @label_status.text = data[:status]
+            @label_position.text = data[:position]
+            @label_head.text = data[:head]
+            @label_artist.text = data[:artist]
+            @label_title.text = data[:title]
+            @label_album.text = data[:album]
+            @label_tgenre.text = data[:tgenre]
+            @label_contact.text = data[:contact]
             @channel_info.show
           end
         }
-        SIGNAL.emit("VstatusChannelinfo", 1)
-        SIGNAL.emit("CgetChannelinfo", url)
+        @sig.emit("VstatusChannelinfo", 1)
+        @sig.emit("CgetChannelinfo", url)
 
 
         @Tchannel = Gtk::timeout_add(2000) {
           puts "Timeout !" if $DEBUG
-          SIGNAL.emit("CgetChannelinfo", url)
+          @sig.emit("CgetChannelinfo", url)
           true
         }
 
@@ -645,17 +680,17 @@ module Geekast
 
 
       def on_channel_info_destroy
-        SIGNAL.handler_block(@SchannelInfo)
-        puts "Timeout channelInfo removed" if $DEBUG
+        @sig.disconnect(@SchannelInfo)
         Gtk::timeout_remove(@Tchannel)
-        SIGNAL.emit("VstatusChannelinfo", 0)
+        puts "Timeout channelInfo removed" if $DEBUG
+        @sig.emit("VstatusChannelinfo", 0)
       end
     end
 
 
 
 
-    
+
     class CallbackPref < GladeBase
       def initialize
         super("geekast.glade", "preferences")
@@ -673,6 +708,18 @@ module Geekast
         # Geekast config
         @checkbutton_player.active = @geekastConfig.internal_player
 
+
+        # Jabber config
+=begin
+        @entry_jid.text = @geekastConfig.jid
+        @entry_jpassword.text = @geekastConfig.jidpass
+        @entry_jressource.text = @geekastConfig.jabressource
+        @entry_jserver.text = @geekastConfig.jabserver
+        @entry_nick.text = @geekastConfig.jabnick
+=end
+
+
+
         unless @geekastConfig.player.empty?
           @filechooserbutton_player.filename = @geekastConfig.player
         else
@@ -686,6 +733,8 @@ module Geekast
         @preferences.destroy
       end
 
+
+      # Peercast config events
       def on_entry_pip_changed(widget)
         @geekastConfig.ip = @entry_pip.text
       end
@@ -702,7 +751,7 @@ module Geekast
         @geekastConfig.yellowpages = @entry_yp.text
       end
 
-
+      # Geekast config events
       def on_filechooserbutton_player_selection_changed
         @geekastConfig.player = @filechooserbutton_player.filename.to_s
       end
@@ -720,26 +769,170 @@ module Geekast
       end
 
 
-
-
-
-
       def on_entry_gpo_changed(widget)
         @geekastConfig.player_options = @entry_gpo.text
       end
 
 
+      # Jabber config events
+      def on_entry_jid_changed
+        @geekastConfig.jid = @entry_jid.text
+      end
+
+      def on_entry_jpassword_changed
+        @geekastConfig.jidpass = @entry_jpassword.text
+      end
+
+      def on_entry_jressource_changed
+        @geekastConfig.jabressource = @entry_jressource.text
+      end
+
+      def on_entry_jserver_changed
+        @geekastConfig.jabserver = @entry_jserver.text
+      end
+
+      def on_entry_nick_changed
+        @geekastConfig.jabnick = @entry_nick.text
+      end
+
+
     end
 
 
 
 
 
+    class Chat < GladeBase
+      attr_reader :status
+
+      def initialize
+        super("geekast.glade", "window_chat")
+        @sig = Sig.instance
+        @window_chat.hide
+        @status = 0
+        nick = Hash.new
+        @stack_page = Array.new
+        @stack_model = Array.new
 
 
 
+        @SchatBuffer = @sig.connect("VjabberBuffer", ['Integer','String', 'Time']) {|channel, buffer, time|
+          puts "VjabberBuffer" if $DEBUG
+          refresh_buffer(channel, buffer, time)
+        }
+        @SchatError = @sig.connect("VjabberError", ['String']) {|data|
+          puts "ERROR : " + data
+          puts "VjabberError" if $DEBUG
+          show_error(data)
+        }
+
+        @SchatRoster = @sig.connect("VjabberRoster", ['Integer','String']) {|channel, data|
+          puts "VjabberRoster" if $DEBUG
+          nick[:nickname] = data
+          @stack_model[channel].model_line(nick)
+        }
+
+        @SchatTopic = @sig.connect("VjabberTopic", ['Integer','String']) {|channel, data|
+          puts "VjabberTopic" if $DEBUG
+          @stack_page[channel]['entry_title'].text = data
+        }
+
+        @SchatNotebook = @sig.connect("VjabberNotebook", ['String']) {|data|
+          puts "VjabberNotebook" if $DEBUG
+          on_notebook_page_append(data)
+        }
+      end
+
+      def on_window_chat_destroy
+        @window_chat.destroy
+      end
+
+      def show
+        @window_chat.show
+      end
+
+      def hide
+        if !@window_chat.destroyed?
+          @window_chat.hide
+        end
+      end
+
+      def on_window_chat_destroy
+        quit
+      end
+
+      def on_toolbutton_quit_clicked
+        destroy
+      end
+
+      def on_toolbutton_disconnect_clicked
+      end
+
+      def on_toolbutton_clear_clicked
+      end
+
+      def on_button_valid_clicked
+        @sig.emit("CjabberChat", @notebook_channel.page, @entry_chat.text)
+        @entry_chat.text = ''
+      end
+
+      def on_entry_chat_activate
+        @sig.emit("CjabberChat", @notebook_channel.page, @entry_chat.text)
+        @entry_chat.text = ''
+      end
+
+
+      def on_notebook_page_append(room)
+        @stack_page.push(GladeXML.new(Config::DATA_DIR + "/glade/geekast.glade", "window_notebookpage"))
+        @stack_page[-1]['window_notebookpage'].hide
+        @stack_page[-1]['vbox_notebook'].unparent
+
+        @stack_model.push(TreeviewRoster.new(@stack_page[-1]['treeview_roster']))
+
+        @notebook_channel.append_page(@stack_page[-1]['vbox_notebook'], tab_label = Gtk::Label.new(room))
+
+        index = @stack_page.rindex(@stack_page[-1])
+        @sig.emit("CjabberConnect", index, room)
+      end
+
+
+      #######
+      private
+      #######
+
+      def refresh_buffer(channel, buffer, time=nil)
+        #        @textview_buffer.buffer.insert(@textview_buffer.buffer.end_iter, "[#{(time || Time.new).getlocal.strftime('%I:%M')}] #{buffer}\n")
+
+        @stack_page[channel]['textview_buffer'].buffer.insert(@stack_page[channel]['textview_buffer'].buffer.end_iter, "[#{(time || Time.new).getlocal.strftime('%I:%M')}] #{buffer}\n")
+
+        #        va = @scrolledwindow_buffer.vadjustment
+        va = @stack_page[channel]['scrolledwindow_buffer'].vadjustment
+
+        va.value = va.upper
+      end
+
+
+      def quit
+        @sig.disconnect(@SchatRoster)
+        @sig.disconnect(@SchatBuffer)
+        @sig.disconnect(@SchatError)
+        @sig.disconnect(@SchatTopic)
+        @sig.disconnect(@SchatNotebook)
+        @status = 1
+      end
+
+      def destroy
+        @window_chat.destroy
+      end
+
+      def show_error(msg)
+        dialog = Gtk::MessageDialog.new(@main, Gtk::Dialog::MODAL, Gtk::MessageDialog::ERROR, Gtk::MessageDialog::BUTTONS_OK, msg)
+        dialog.signal_connect("response") { dialog.destroy }
+        dialog.run
+      end
+    end
+
 
 
-    
   end
 end
