@@ -1,4 +1,4 @@
-#  interface.rb - 
+#  interface.rb -
 #  Copyright (C) 2004-2006 Frédéric Logier
 
 # This program is free software; you can redistribute it and/or modify
@@ -18,9 +18,12 @@
 require 'net/http'
 require 'geekast/views/lib/treeview'
 require 'geekast/views/lib/glade_base'
-require 'gtktrayicon'
 require 'gtkmozembed'
 
+# Sig
+require 'geekast/lib/signal'
+
+
 module Geekast
   module View
 
@@ -34,43 +37,43 @@ module Geekast
 
         @treeview = treeview
 
-        @model = Gtk::ListStore.new(String, String, 
+        @model = Gtk::ListStore.new(String, String,
                                     Integer, String,
-                                    String,  String, 
-                                    String, Integer, 
-                                    String, Integer, 
+                                    String,  String,
+                                    String, Integer,
+                                    String, Integer,
                                     Integer)
 
         @treeview_ColumnType = {
-          "name" => "text", 
-          "id" => "text",
-          "bitrate" => "int",
-          "type" => "text",
-          "genre" => "text",
-          "desc" => "text",
-          "url" => "text",
-          "uptime" => "int",
-          "comment" => "text",
-          "skips" => "int",
-          "age" => "int"
+          :name => "text",
+          :id => "text",
+          :bitrate => "int",
+          :type => "text",
+          :genre => "text",
+          :desc => "text",
+          :url => "text",
+          :uptime => "int",
+          :comment => "text",
+          :skips => "int",
+          :age => "int"
         }
 
         @treeview_ColumnNum = {
-          "name" => 0, 
-          "id" => 1,
-          "bitrate" => 2,
-          "type" => 3,
-          "genre" => 4,
-          "desc" => 5,
-          "url" => 6,
-          "uptime" => 7,
-          "comment" => 8,
-          "skips" => 9,
-          "age" => 10
+          :name => 0,
+          :id => 1,
+          :bitrate => 2,
+          :type => 3,
+          :genre => 4,
+          :desc => 5,
+          :url => 6,
+          :uptime => 7,
+          :comment => 8,
+          :skips => 9,
+          :age => 10
         }
-        
 
-        # Filter station according to the filter toolbar widgets. 
+
+        # Filter station according to the filter toolbar widgets.
         @filtered_model = Gtk::TreeModelFilter.new(@model)
 
 
@@ -101,7 +104,7 @@ module Geekast
       def change_filter(col, filter)
         @col = col
         @filter = filter
-        @filtered_model.refilter  
+        @filtered_model.refilter
         #        puts "filter = " + @filter
       end
 
@@ -115,29 +118,29 @@ module Geekast
       def initialize(treeview)
         @treeview = treeview
 
-        @model = Gtk::ListStore.new(String, String, 
-                                    String, String, 
-                                    Integer, String, 
+        @model = Gtk::ListStore.new(String, String,
+                                    String, String,
+                                    Integer, String,
                                     String)
 
         @treeview_ColumnType = {
-          "name" => "text", 
-          "type" => "text",
-          "url" => "text",
-          "id" => "text",
-          "bitrate" => "int",
-          "desc" => "text",
-          "genre" => "text"
+          :name => "text",
+          :type => "text",
+          :url => "text",
+          :id => "text",
+          :bitrate => "int",
+          :desc => "text",
+          :genre => "text"
         }
-        
+
         @treeview_ColumnNum = {
-          "name" => 0, 
-          "type" => 1,
-          "url" => 2,
-          "id" => 3,
-          "bitrate" => 4,
-          "desc" => 5, 
-          "genre" => 6
+          :name => 0,
+          :type => 1,
+          :url => 2,
+          :id => 3,
+          :bitrate => 4,
+          :desc => 5,
+          :genre => 6
         }
 
         @channel_model = @model
@@ -152,26 +155,26 @@ module Geekast
       def initialize(treeview)
         @treeview = treeview
 
-        @model = Gtk::ListStore.new(String, String, 
-                                    String, String, 
+        @model = Gtk::ListStore.new(String, String,
+                                    String, String,
                                     String, String)
 
         @treeview_ColumnType = {
-          "agent" => "text",
-          "ipport" => "text",
-          "kbits" => "text",
-          "status" => "text",
-          "time" => "text",
-          "type" => "text"
+          :agent => "text",
+          :ipport => "text",
+          :kbits => "text",
+          :status => "text",
+          :time => "text",
+          :type => "text"
         }
 
         @treeview_ColumnNum = {
-          "agent" => 0,
-          "ipport" => 1,
-          "kbits" => 2,
-          "status" => 3,
-          "time" => 4,
-          "type" => 5
+          :agent => 0,
+          :ipport => 1,
+          :kbits => 2,
+          :status => 3,
+          :time => 4,
+          :type => 5
         }
 
         @channel_model = @model
@@ -182,7 +185,7 @@ module Geekast
 
 
   class TreeviewInfoInformation < TreeviewLib
-      
+
       def initialize(treeview)
         @treeview = treeview
 
@@ -190,17 +193,17 @@ module Geekast
                                     String, String)
 
         @treeview_ColumnType = {
-          "uptime" => "text", 
-          "channel_cache" => "text",
-          "firewalled" => "text",
-          "gnethostcache" => "text",
+          :uptime => "text",
+          :channel_cache => "text",
+          :firewalled => "text",
+          :gnethostcache => "text",
         }
 
         @treeview_ColumnNum = {
-          "uptime" => 0, 
-          "channel_cache" => 1,
-          "firewalled" => 2,
-          "gnethostcache" => 3
+          :uptime => 0,
+          :channel_cache => 1,
+          :firewalled => 2,
+          :gnethostcache => 3
         }
 
         @channel_model = @model
@@ -209,25 +212,25 @@ module Geekast
     end
 
   class TreeviewInfoBandwidth < TreeviewLib
-      
+
       def initialize(treeview)
         @treeview = treeview
 
-        @model = Gtk::ListStore.new(String, String, 
+        @model = Gtk::ListStore.new(String, String,
                                     String, String)
 
         @treeview_ColumnType = {
-          "name" => "text", 
-          "id" => "text",
-          "bitrate" => "text",
-          "type" => "text",
+          :name => "text",
+          :id => "text",
+          :bitrate => "text",
+          :type => "text",
         }
 
         @treeview_ColumnNum = {
-          "name" => 0, 
-          "id" => 1,
-          "bitrate" => 2,
-          "type" => 3,
+          :name => 0,
+          :id => 1,
+          :bitrate => 2,
+          :type => 3,
         }
 
         @channel_model = @model
@@ -236,21 +239,43 @@ module Geekast
     end
 
   class TreeviewInfoConnections < TreeviewLib
-       
+
       def initialize(treeview)
         @treeview = treeview
 
-        @model = Gtk::ListStore.new(String, String) 
+        @model = Gtk::ListStore.new(String, String)
 
 
         @treeview_ColumnType = {
-          "name" => "text", 
-          "id" => "text",
+          :name => "text",
+          :id => "text",
         }
 
         @treeview_ColumnNum = {
-          "name" => 0, 
-          "id" => 1,
+          :name => 0,
+          :id => 1,
+        }
+
+        @channel_model = @model
+        super()
+      end
+    end
+
+
+
+  class TreeviewRoster < TreeviewLib
+
+      def initialize(treeview)
+        @treeview = treeview
+
+        @model = Gtk::ListStore.new(String)
+
+        @treeview_ColumnType = {
+          :nickname => "text"
+        }
+
+        @treeview_ColumnNum = {
+          :nickname => 0
         }
 
         @channel_model = @model
@@ -262,12 +287,14 @@ module Geekast
 
 
 
-    
+
+
     class ContextualMenu < GladeBase
-      
+
       def initialize(treeview_init, menu_glade, url, model)
         puts "context ok" if $DEBUG
         super("geekast.glade", menu_glade)
+        @sig = Sig.instance
         @treeview = treeview_init
         @menu = instance_eval("@" + menu_glade.intern.to_s)
         @id
@@ -280,16 +307,35 @@ module Geekast
       end
 
 
-      def on_chat_activate
+      def on_webchat_activate
         get_id
-        url = "http://" + Config::YELLOWPAGES_IP + ":" + Config::YELLOWPAGES_PORT + "/chat.php?cid=" + @id.to_s
-        SIGNAL.emit("CrefreshWeb", url)
+        url = "http://" + Config::YELLOWPAGES_IP + ":" + Config::YELLOWPAGES_PORT + "/getchat?id=" + @id.to_s
+        @sig.emit("CrefreshWeb", url)
+      end
+
+      def on_jabberchat_activate
+        liststore = @treeview.model
+        rowselected = @treeview.selection
+
+        # liststore.each do |model,path,iter|
+        #  printf("Row %s: %s\n", path.to_str, iter[3]);
+        # end
+        rowselected.selected_each do |model,path,iter|
+          # printf("Row %s: %s\n", path.to_str, iter[@model.treeview_ColumnNum[:name]])
+          name = iter[@model.treeview_ColumnNum[:name]]
+          #          printf("name : %s\n", name.tr(' ',''))
+
+          channel = name.tr(' ','').tr('/','').tr('@','').strip
+
+          @sig.emit("VjabberNotebook", channel[0..20])
+        end
+
       end
 
       def on_stats_activate
         get_id
-        url = "http://" + Config::YELLOWPAGES_IP + ":" + Config::YELLOWPAGES_PORT + "/getgmt.php?id=" + @id.to_s
-        SIGNAL.emit("CrefreshWeb", url)
+        url = "http://" + Config::YELLOWPAGES_IP + ":" + Config::YELLOWPAGES_PORT + "/getgmt?id=" + @id.to_s
+        @sig.emit("CrefreshWeb", url)
       end
 
 
@@ -304,26 +350,26 @@ module Geekast
         # end
 
         rowselected.selected_each do |model,path,iter|
-      
+
           flux = {"channel" =>
-              {"name" => iter[@model.treeview_ColumnNum['name']],
-              "url" => iter[@model.treeview_ColumnNum['url']],       
-              "type" => iter[@model.treeview_ColumnNum['type']], 
-              "id" => iter[@model.treeview_ColumnNum['id']],
-              "desc" => iter[@model.treeview_ColumnNum['desc']],
-              "bitrate" => iter[@model.treeview_ColumnNum['bitrate']],
-              "genre" => iter[@model.treeview_ColumnNum['genre']]
+              {:name => iter[@model.treeview_ColumnNum[:name]],
+              :url => iter[@model.treeview_ColumnNum[:url]],
+              :type => iter[@model.treeview_ColumnNum[:type]],
+              :id => iter[@model.treeview_ColumnNum[:id]],
+              :desc => iter[@model.treeview_ColumnNum[:desc]],
+              :bitrate => iter[@model.treeview_ColumnNum[:bitrate]],
+              :genre => iter[@model.treeview_ColumnNum[:genre]]
             }
           }
-          SIGNAL.emit("CbookmarkAdd", flux)
-          SIGNAL.emit("CrefreshNB", Config::BOOKMARK)
+          @sig.emit("CbookmarkAdd", flux)
+          @sig.emit("CrefreshNB", Config::BOOKMARK)
         end
 
       end
 
 
 
-      
+
       def on_play_activate
         @geekastConfig = Geekast::Model::Preferences.instance
 
@@ -331,29 +377,29 @@ module Geekast
         if !@@url
           liststore = @treeview.model
           rowselected = @treeview.selection
-          
+
           rowselected.selected_each do |model,path,iter|
-            printf("Row %s: %s\n", path.to_str, iter[@model.treeview_ColumnNum['id']]) if $DEBUG
-            
+            printf("Row %s: %s\n", path.to_str, iter[@model.treeview_ColumnNum[:id]]) if $DEBUG
+
             ####################################################
-            @@url = "http://" + @geekastConfig.ip + ":" + @geekastConfig.port.to_i.to_s + "/stream/" +  iter[@model.treeview_ColumnNum['id']]
+            @@url = "http://" + @geekastConfig.ip + ":" + @geekastConfig.port.to_i.to_s + "/stream/" +  iter[@model.treeview_ColumnNum[:id]]
             #          url = "http://" + Geekast::Config::PEERCAST_IP
             #          url = "http://127.0.0.1"
             # link = "/stream/" +  iter[3]
             ####################################################
           end
         end
-        
+
         if @@url
           if @geekastConfig.internal_player == FALSE
             Thread.new {
               system(@geekastConfig.player  + " " + @geekastConfig.player_options + " " + @@url)
             }
-          else 
+          else
             @multimedia.create(@@url)
-          end   
+          end
         end
-        @@url = nil       
+        @@url = nil
       end
 
       def on_web_activate
@@ -364,13 +410,15 @@ module Geekast
         #  printf("Row %s: %s\n", path.to_str, iter[3]);
         # end
         rowselected.selected_each do |model,path,iter|
-          printf("Row %s: %s\n", path.to_str, iter[@model.treeview_ColumnNum['url']]) if $DEBUG
-          url = iter[@model.treeview_ColumnNum['url']]
+          printf("Row %s: %s\n", path.to_str, iter[@model.treeview_ColumnNum[:url]]) if $DEBUG
+          url = iter[@model.treeview_ColumnNum[:url]]
           printf("url : %s\n", url) if $DEBUG
 
-          Thread.new {
-            system("gnome-open " + '"' + url + '"')
-          }
+          if url
+            Thread.new {
+              system("gnome-open " + '"' + url + '"')
+            }
+          end
         end
       end
 
@@ -391,7 +439,7 @@ module Geekast
         get_id
         url = "/admin?cmd=stop&id=" + @id.to_s
         http_command(url)
-        SIGNAL.emit("CrefreshNB", Config::RELAY)
+        @sig.emit("CrefreshNB", Config::RELAY)
       end
 
 
@@ -401,7 +449,7 @@ module Geekast
         ChannelInfo.new(url)
       end
 
-      
+
       def popup
         # Popup the menu on right click
         @treeview.signal_connect("button_press_event") do |widget, event|
@@ -409,14 +457,14 @@ module Geekast
             @menu.popup(nil, nil, event.button, event.time)
           end
         end
-        
+
         # Popup the menu on Shift-F10
         @treeview.signal_connect("popup_menu") { @menu.popup(
-                                                             nil, 
-                                                             nil, 
-                                                             0, 
+                                                             nil,
+                                                             nil,
+                                                             0,
                                                              Gdk::Event::CURRENT_TIME) }
-        
+
       end
 
 
@@ -439,22 +487,22 @@ module Geekast
         end
 
         # flush bookmark
-        SIGNAL.emit("CbookmarkDel", 0)
+        @sig.emit("CbookmarkDel", 0)
 
         liststore.each do |model,path,iter|
           flux = {"channel" =>
-              {"name" => iter[@model.treeview_ColumnNum['name']],
-              "url" => iter[@model.treeview_ColumnNum['url']],       
-              "type" => iter[@model.treeview_ColumnNum['type']], 
-              "id" => iter[@model.treeview_ColumnNum['id']],
-              "desc" => iter[@model.treeview_ColumnNum['desc']],
-              "bitrate" => iter[@model.treeview_ColumnNum['bitrate']],
-              "genre" => iter[@model.treeview_ColumnNum['genre']]
+              {:name => iter[@model.treeview_ColumnNum[:name]],
+              :url => iter[@model.treeview_ColumnNum[:url]],
+              :type => iter[@model.treeview_ColumnNum[:type]],
+              :id => iter[@model.treeview_ColumnNum[:id]],
+              :desc => iter[@model.treeview_ColumnNum[:desc]],
+              :bitrate => iter[@model.treeview_ColumnNum[:bitrate]],
+              :genre => iter[@model.treeview_ColumnNum[:genre]]
             }
           }
-          SIGNAL.emit("CbookmarkAdd", flux)
+          @sig.emit("CbookmarkAdd", flux)
         end
-        SIGNAL.emit("CrefreshNB", Config::BOOKMARK)
+        @sig.emit("CrefreshNB", Config::BOOKMARK)
       end
 
 
@@ -465,7 +513,7 @@ module Geekast
       def get_id
         liststore = @treeview.model
         rowselected = @treeview.selection
-        
+
         # liststore.each do |model,path,iter|
         #  printf("Row %s: %s\n", path.to_str, iter[3]);
         # end
@@ -473,8 +521,8 @@ module Geekast
           #          printf("Row %s: %s\n", path.to_str, iter[5]);
           #          url = "http://localhost:7144/admin?cmd=keep&id=" +  iter[5]
           # printf("url : %s\n", url)
-          
-          id = @model.treeview_ColumnNum['id']
+
+          id = @model.treeview_ColumnNum[:id]
 
           @id = iter[id]
         end
@@ -490,10 +538,10 @@ module Geekast
 
         begin
           puts url if $DEBUG
-          
+
           req = Net::HTTP::Get.new(url)
           req.basic_auth('', @geekastConfig.password)
-          @data = @peercast_connect.request(req).body          
+          @data = @peercast_connect.request(req).body
         # @resp, @data = @peercast_connect.get(url, nil )
         rescue
           puts "http command error"
@@ -505,11 +553,11 @@ module Geekast
 
 
     class Connections
-      
+
       def initialize
       end
 
-      
+
       def refresh_connection(url)
       @geekastConfig = Geekast::Model::Preferences.instance
 
@@ -518,10 +566,10 @@ module Geekast
                                               @geekastConfig.port.to_i.to_s)
 
 
-      
+
         begin
           puts url if $DEBUG
-          
+
           @resp, @data = @peercast_connections.get(url, nil )
         rescue
           puts "http refresh connections error"
@@ -540,42 +588,23 @@ module Geekast
         super("geekast.glade", "menu_trayicon")
         @geekastConfig = Geekast::Model::Preferences.instance
         @multimedia = Multimedia.main
+        @sig = Sig.instance
 
         @hItem = Hash.new
-        @aItemRC = Array.new
-        @aItemYP = Array.new
-        @aItemBK = Array.new
+        @aItemRC, @aItemYP, @aItemBK = [], [], []
 
-        @url = "http://" + @geekastConfig.ip + ":" + @geekastConfig.port.to_i.to_s + "/stream/"
-       
-        @check_clic = 0
+        @url = "http://#{@geekastConfig.ip}:#{@geekastConfig.port.to_i.to_s}/stream/"
 
-        @tray = Gtk::TrayIcon.new("geekast")
-
-        #        @tray.add(Gtk::Image.new(Gtk::Stock::OPEN, Gtk::IconSize::SMALL_TOOLBAR))
-
-        icon = Gtk::Image.new(Config::DATA_DIR + "/pixmaps/small-icon.gif")
-        event_box = Gtk::EventBox.new.add(icon)
-        event_box.signal_connect("button_press_event") do |widget, event|
-          if event.kind_of? Gdk::EventButton and event.button == 1
-            OnClick(main_window)
-          elsif event.kind_of? Gdk::EventButton and event.button == 3
-            @menu_trayicon.popup(nil, nil, event.button, event.time)
-          end
-        end
+        @tray = Gtk::StatusIcon.new
+        icon = Gdk::Pixbuf.new(Config::DATA_DIR + "/pixmaps/small-icon.gif")
+        @tray.set_pixbuf(icon)
+        @tray.signal_connect("activate"){
+          !main_window.visible? == true ? main_window.show : main_window.hide
+        }
+        @tray.signal_connect("popup-menu"){|w, button, activate_time|
+          @menu_trayicon.popup(nil, nil, button, activate_time)
+        }
         init_bookmark
-        @tray.add(event_box)
-        @tray.show_all
-      end
-
-      def OnClick(main_window)
-        if @check_clic == 0 then
-          main_window.hide
-          @check_clic = 1
-        else
-          main_window.show
-          @check_clic = 0
-        end
       end
 
       def on_stop_clicked
@@ -593,19 +622,17 @@ module Geekast
         @item = @menu_trayicon.active
         clean(Config::RELAY)
 
-        SIGNAL.emit("CrefreshNB", Config::RELAY)
+        @sig.emit("CrefreshNB", Config::RELAY)
       end
 
       def on_refresh_yp_activate
         @item = @menu_trayicon.active
         clean(Config::YP)
 
-        SIGNAL.emit("CrefreshNB", Config::YP)
+        @sig.emit("CrefreshNB", Config::YP)
       end
 
 
-
-
       def clean(notebook)
         case notebook
         when Config::RELAY
@@ -629,47 +656,47 @@ module Geekast
             @hItem[@item].clear
           else
             @hItem[@item] = @aItemYP
-          end          
+          end
         end
       end
 
 
       def append(channel)
         message("refresh")
-        
         menu = @item.submenu
-        item = Gtk::MenuItem.new(channel["name"])
+        item = Gtk::MenuItem.new(channel[:name])
         @hItem[@item].push(item)
 
         menu.append(item)
         item.signal_connect("activate") {
-          url = @url + channel["id"]
+          url = @url + channel[:id]
           if @geekastConfig.internal_player == FALSE
             Thread.new {
               system(@geekastConfig.player  + " " + @geekastConfig.player_options + " " + url)
             }
-          else 
+          else
             @multimedia.create(url)
-          end   
+          end
         }
         menu.show_all
       end
 
 
       def message(msg)
-        @id_msg = @tray.send_message(10000, msg)
+        @tray.set_tooltip(msg)
       end
 
       def cancel
         @tray.cancel_message(@id_msg)
       end
       def on_quit_activate
+        @multimedia.stop
         Gtk.main_quit
       end
 
     end
 
 
-    
+
   end
 end
