@@ -1,5 +1,5 @@
 #  view.rb - main script
-#  Copyright (C) 2004-2006 Frédéric Logier
+#  Copyright (C) 2004-2007 Frédéric Logier
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@ module Geekast
     private
     #######
 
-    class Interface
+    class Interface < Sig
       include Singleton
 
       def initialize
